/*!
 * rivet-core - @version 2.0.0-alpha.5

 * Copyright (C) 2018 The Trustees of Indiana University
 * SPDX-License-Identifier: BSD-3-Clause
 */

var Rivet = (function (exports) {
  'use strict';

  if (!Element.prototype.closest) {
    if (!Element.prototype.matches) {
      Element.prototype.matches = Element.prototype.msMatchesSelector || Element.prototype.webkitMatchesSelector;
    }

    Element.prototype.closest = function (s) {
      var el = this;
      var ancestor = this;
      if (!document.documentElement.contains(el)) return null;

      do {
        if (ancestor.matches(s)) return ancestor;
        ancestor = ancestor.parentElement;
      } while (ancestor !== null);

      return null;
    };
  }

  (function () {
    if (typeof window.CustomEvent === 'function') return false;

    function CustomEvent(event, params) {
      params = params || {
        bubbles: false,
        cancelable: false,
        detail: undefined
      };
      var evt = document.createEvent('CustomEvent');
      evt.initCustomEvent(event, params.bubbles, params.cancelable, params.detail);
      return evt;
    }

    CustomEvent.prototype = window.Event.prototype;
    window.CustomEvent = CustomEvent;
  })();

  (function (arr) {
    arr.forEach(function (item) {
      if (item.hasOwnProperty('remove')) {
        return;
      }


      Object.defineProperty(item, 'remove', {
        configurable: true,
        enumerable: true,
        writable: true,
        value: function remove() {
          if (this.parentNode === null) {
            return;
          }

          this.parentNode.removeChild(this);
        }
      });
    });
  })([Element.prototype, CharacterData.prototype, DocumentType.prototype]);

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    return Constructor;
  }

  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function");
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        writable: true,
        configurable: true
      }
    });
    if (superClass) _setPrototypeOf(subClass, superClass);
  }

  function _getPrototypeOf(o) {
    _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
      return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf(o);
  }

  function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
      o.__proto__ = p;
      return o;
    };

    return _setPrototypeOf(o, p);
  }

  function _assertThisInitialized(self) {
    if (self === void 0) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return self;
  }

  function _possibleConstructorReturn(self, call) {
    if (call && (typeof call === "object" || typeof call === "function")) {
      return call;
    }

    return _assertThisInitialized(self);
  }

  var globalSettings = {
    prefix: 'rvt'
  };

  var Lie = typeof Promise === 'function' ? Promise : function (fn) {
    let queue = [],
        resolved = 0,
        value;
    fn($ => {
      value = $;
      resolved = 1;
      queue.splice(0).forEach(then);
    });
    return {
      then
    };

    function then(fn) {
      return resolved ? setTimeout(fn, 0, value) : queue.push(fn), this;
    }
  };

  const {
    document: document$1,
    MutationObserver: MutationObserver$1,
    Set: Set$1,
    WeakMap: WeakMap$1
  } = self;

  const elements = element => 'querySelectorAll' in element;

  const {
    filter
  } = [];
  var QSAO = (options => {
    const live = new WeakMap$1();

    const callback = records => {
      const {
        query
      } = options;

      if (query.length) {
        for (let i = 0, {
          length
        } = records; i < length; i++) {
          loop(filter.call(records[i].addedNodes, elements), true, query);
          loop(filter.call(records[i].removedNodes, elements), false, query);
        }
      }
    };

    const drop = elements => {
      for (let i = 0, {
        length
      } = elements; i < length; i++) live.delete(elements[i]);
    };

    const flush = () => {
      callback(observer.takeRecords());
    };

    const loop = (elements, connected, query, set = new Set$1()) => {
      for (let selectors, element, i = 0, {
        length
      } = elements; i < length; i++) {
        if (!set.has(element = elements[i])) {
          set.add(element);

          if (connected) {
            for (let q, m = matches(element), i = 0, {
              length
            } = query; i < length; i++) {
              if (m.call(element, q = query[i])) {
                if (!live.has(element)) live.set(element, new Set$1());
                selectors = live.get(element); 

                if (!selectors.has(q)) {
                  selectors.add(q);
                  options.handle(element, connected, q);
                }
              }
            }
          } 
          else if (live.has(element)) {
              selectors = live.get(element);
              live.delete(element);
              selectors.forEach(q => {
                options.handle(element, connected, q);
              });
            }

          loop(querySelectorAll(element), connected, query, set);
        }
      }
    };

    const matches = element => element.matches || element.webkitMatchesSelector || element.msMatchesSelector;

    const parse = (elements, connected = true) => {
      loop(elements, connected, options.query);
    };

    const querySelectorAll = root => query.length ? root.querySelectorAll(query) : query;

    const observer = new MutationObserver$1(callback);
    const root = options.root || document$1;
    const {
      query
    } = options;
    observer.observe(root, {
      childList: true,
      subtree: true
    });
    parse(querySelectorAll(root));
    return {
      drop,
      flush,
      observer,
      parse
    };
  });

  const {
    create,
    keys
  } = Object;
  const attributes = new WeakMap();
  const lazy = new Set();
  const query = [];
  const config = {};
  const defined = {};

  const attributeChangedCallback = (records, o) => {
    for (let h = attributes.get(o), i = 0, {
      length
    } = records; i < length; i++) {
      const {
        target,
        attributeName,
        oldValue
      } = records[i];
      const newValue = target.getAttribute(attributeName);
      h.attributeChanged(attributeName, oldValue, newValue);
    }
  };

  const set = (value, m, l, o) => {
    const handler = create(o, {
      element: {
        enumerable: true,
        value
      }
    });

    for (let i = 0, {
      length
    } = l; i < length; i++) value.addEventListener(l[i].t, handler, l[i].o);

    m.set(value, handler);
    if (handler.init) handler.init();
    const {
      observedAttributes
    } = o;

    if (observedAttributes) {
      const mo = new MutationObserver(attributeChangedCallback);
      mo.observe(value, {
        attributes: true,
        attributeOldValue: true,
        attributeFilter: observedAttributes.map(attributeName => {
          if (value.hasAttribute(attributeName)) handler.attributeChanged(attributeName, null, value.getAttribute(attributeName));
          return attributeName;
        })
      });
      attributes.set(mo, handler);
    }

    return handler;
  };

  const {
    drop,
    flush,
    parse
  } = QSAO({
    query,

    handle(element, connected, selector) {
      const {
        m,
        l,
        o
      } = config[selector];
      const handler = m.get(element) || set(element, m, l, o);
      const method = connected ? 'connected' : 'disconnected';
      if (method in handler) handler[method]();
    }

  });
  const define = (selector, definition) => {
    if (-1 < query.indexOf(selector)) throw new Error('duplicated: ' + selector);
    flush();
    const listeners = [];
    const retype = create(null);

    for (let k = keys(definition), i = 0, {
      length
    } = k; i < length; i++) {
      const key = k[i];

      if (/^on/.test(key) && !/Options$/.test(key)) {
        const options = definition[key + 'Options'] || false;
        const lower = key.toLowerCase();
        let type = lower.slice(2);
        listeners.push({
          t: type,
          o: options
        });
        retype[type] = key;

        if (lower !== key) {
          type = key.slice(2, 3).toLowerCase() + key.slice(3);
          retype[type] = key;
          listeners.push({
            t: type,
            o: options
          });
        }
      }
    }

    if (listeners.length) {
      definition.handleEvent = function (event) {
        this[retype[event.type]](event);
      };
    }

    query.push(selector);
    config[selector] = {
      m: new WeakMap(),
      l: listeners,
      o: definition
    };
    parse(document.querySelectorAll(selector));
    whenDefined(selector);
    if (!lazy.has(selector)) defined[selector]._();
  };
  const whenDefined = selector => {
    if (!(selector in defined)) {
      let _,
          $ = new Lie($ => {
        _ = $;
      });

      defined[selector] = {
        _,
        $
      };
    }

    return defined[selector].$;
  };

  var Component =
  function () {
    function Component() {
      _classCallCheck(this, Component);
    }

    _createClass(Component, null, [{
      key: "initAll",
      value: function initAll() {
        this.init(this.selector);
      }
    }, {
      key: "init",
      value: function init(selector) {
        define(selector, this.methods);
        return document.querySelector(selector);
      }
    }, {
      key: "bindMethodToDOMElement",
      value: function bindMethodToDOMElement(self, name, method) {
        Object.defineProperty(self.element, name, {
          value: method.bind(self),
          writable: false
        });
      }
    }, {
      key: "dispatchCustomEvent",
      value: function dispatchCustomEvent(eventName, element, detail) {
        var prefix = globalSettings.prefix;
        var event = new CustomEvent("".concat(prefix, ":").concat(eventName), {
          bubbles: true,
          cancelable: true,
          detail: detail
        });
        return element.dispatchEvent(event);
      }
    }, {
      key: "dispatchComponentAddedEvent",
      value: function dispatchComponentAddedEvent(element) {
        return this.dispatchCustomEvent('componentAdded', document, {
          component: element
        });
      }
    }, {
      key: "dispatchComponentRemovedEvent",
      value: function dispatchComponentRemovedEvent(element) {
        return this.dispatchCustomEvent('componentRemoved', document, {
          component: element
        });
      }
    }, {
      key: "selector",
      get: function get() {
      }
    }, {
      key: "methods",
      get: function get() {
      }
    }]);

    return Component;
  }();


  var nodeListToArray = function nodeListToArray(nodes) {
    return Array.prototype.slice.call(nodes);
  };

  var keyCodes = {
    up: 38,
    down: 40,
    left: 37,
    right: 39,
    tab: 9,
    enter: 13,
    escape: 27,
    home: 36,
    end: 35,
    pageUp: 33,
    pageDown: 34
  };

  var Accordion =
  function (_Component) {
    _inherits(Accordion, _Component);

    function Accordion() {
      _classCallCheck(this, Accordion);

      return _possibleConstructorReturn(this, _getPrototypeOf(Accordion).apply(this, arguments));
    }

    _createClass(Accordion, null, [{
      key: "selector",
      get: function get() {
        return '[data-rvt-accordion]';
      }
    }, {
      key: "methods",
      get: function get() {
        return {
          init: function init() {
            var _this = this;

            console.log('Accordion::init()');
            this.openAllOnInit = this.element.hasAttribute('data-rvt-accordion-open-all');
            this.triggerAttribute = 'data-rvt-accordion-trigger';
            this.triggerSelector = "[".concat(this.triggerAttribute, "]");
            this.triggers = nodeListToArray(this.element.querySelectorAll(this.triggerSelector));

            if (this.triggers.length < 2) {
              console.warn('An accordion should contain *at least two* accordion triggers with the "data-rvt-accordion-trigger" attribute');
            }

            this.panelAttribute = 'data-rvt-accordion-panel';
            this.panelSelector = "[".concat(this.panelAttribute, "]");
            this.panels = nodeListToArray(this.element.querySelectorAll(this.panelSelector));
            var initializedPanels = this.element.querySelectorAll("[data-rvt-accordion-panel-init]");

            try {
              if (initializedPanels.length > 1) {
                console.warn('Caught');
              }

              var initialPanel; 

              this.panels.forEach(function (panel, index) {
                if (panel.hasAttribute('data-rvt-accordion-panel-init')) {
                  initialPanel = panel;
                } else {
                  _this.panels[index].setAttribute('hidden', '');
                }
              }); 

              this.openOnInit = initialPanel.getAttribute(this.panelAttribute);
            } catch (e) {
              console.warn('Only one accordion panel should have the data-rvt-accordion-panel-init attribute. If you wish to open all panels on initialization, please apply the appropriate attribute to the data-rvt-accordion element');
            } 


            Component.bindMethodToDOMElement(this, 'open', this.open);
            Component.bindMethodToDOMElement(this, 'close', this.close);
            this._handleClick = this._handleClick.bind(this);
            this._handleKeydown = this._handleKeydown.bind(this);
          },
          connected: function connected() {
            Component.dispatchComponentAddedEvent(this.element);

            this._openOnInit();
          },
          disconnected: function disconnected() {
            Component.dispatchComponentRemovedEvent(this.element);
          },
          onClick: function onClick(event) {
            this._handleClick(event);
          },
          onKeydown: function onKeydown(event) {
            this._handleKeydown(event);
          },
          _openOnInit: function _openOnInit() {
            var _this2 = this;

            if (this.openAllOnInit === true) {
              this.panels.forEach(function (panel) {
                _this2.open(panel.getAttribute(_this2.panelAttribute));
              });
            } else {
              this.panels.forEach(function (panel, index) {
                if (panel.getAttribute(_this2.panelAttribute) !== _this2.openOnInit) {
                  _this2.panels[index].setAttribute('hidden', '');

                  var trigger = _this2.element.querySelector("[".concat(_this2.triggerAttribute, "=\"").concat(panel.dataset.rvtAccordionPanel, "\"]")); 


                  trigger.setAttribute('aria-expanded', 'false');
                } else {
                  _this2.open(_this2.openOnInit);
                }
              });
            }
          },
          _handleClick: function _handleClick(event) {
            var currentTrigger = event.target.closest(this.triggerSelector); 

            if (!currentTrigger) return; 

            var currentTriggerValue = currentTrigger.getAttribute(this.triggerAttribute); 

            currentTrigger.getAttribute('aria-expanded') === 'true' ? this.close(currentTriggerValue) : this.open(currentTriggerValue);
          },
          _handleKeydown: function _handleKeydown(event) {
            if (event.keyCode === keyCodes.up || event.keyCode === keyCodes.down || event.keyCode === keyCodes.end || event.keyCode === keyCodes.home) {
              var accordionParent = event.target.closest('[data-rvt-accordion]'); 

              if (!accordionParent) return;
              var currentTrigger = event.target.closest(this.triggerSelector);
              if (!currentTrigger) return; 

              var nextTrigger = this.triggers.indexOf(currentTrigger) + 1;
              var prevTrigger = this.triggers.indexOf(currentTrigger) - 1;

              switch (event.keyCode) {
                case keyCodes.up:
                  {
                    if (this.triggers[prevTrigger] === undefined) {
                      this.triggers[this.triggers.length - 1].focus();
                    } else {
                      this.triggers[prevTrigger].focus();
                    }

                    break;
                  }

                case keyCodes.down:
                  {
                    if (this.triggers[nextTrigger] === undefined) {
                      this.triggers[0].focus();
                    } else {
                      this.triggers[nextTrigger].focus();
                    }

                    break;
                  }

                case keyCodes.end:
                  {
                    this.triggers[this.triggers.length - 1].focus();
                    break;
                  }

                case keyCodes.home:
                  {
                    this.triggers[0].focus();
                    break;
                  }
              }
            }
          },
          open: function open(panelId) {
            var panel = this.element.querySelector("[".concat(this.panelAttribute, "=\"").concat(panelId, "\"]"));

            if (!panel) {
              console.warn("No such panel '".concat(panelId, "' in Accordion.open()"));
              return;
            }

            var activationEvent = Component.dispatchCustomEvent('accordionOpened', panel, {
              id: panel.dataset.rvtAccordionPanel
            });
            if (!activationEvent) return;
            var trigger = this.element.querySelector("[".concat(this.triggerAttribute, " = \"").concat(panel.dataset.rvtAccordionPanel, "\"]"));
            console.log("[".concat(this.triggerAttribute, " = \"").concat(panel.dataset.rvtAccordionPanel, "\"]")); 

            panel.removeAttribute('hidden');
            trigger.setAttribute('aria-expanded', 'true');
          },
          close: function close(panelId) {
            var panel = this.element.querySelector("[".concat(this.panelAttribute, "=\"").concat(panelId, "\"]"));

            if (!panel) {
              console.warn("No such panel '".concat(panelId, "' in Accordion.close()"));
              return;
            }

            var activationEvent = Component.dispatchCustomEvent('accordionClosed', panel, {
              id: panel.dataset.rvtAccordionPanel
            });
            if (!activationEvent) return;
            var trigger = this.element.querySelector("[".concat(this.triggerAttribute, " = \"").concat(panel.dataset.rvtAccordionPanel, "\"]")); 

            panel.setAttribute('hidden', '');
            trigger.setAttribute('aria-expanded', 'false');
          }
        };
      }
    }]);

    return Accordion;
  }(Component);

  var Alert =
  function (_Component) {
    _inherits(Alert, _Component);

    function Alert() {
      _classCallCheck(this, Alert);

      return _possibleConstructorReturn(this, _getPrototypeOf(Alert).apply(this, arguments));
    }

    _createClass(Alert, null, [{
      key: "selector",
      get: function get() {
        return '[data-rvt-alert]';
      }
    }, {
      key: "methods",
      get: function get() {
        return {
          init: function init() {
            console.log('Alert::init()');
            this.closeAttribute = 'data-rvt-alert-close';
            this.closeSelector = "[".concat(this.closeAttribute, "]");
            this.closeButton = null;
            Component.bindMethodToDOMElement(this, 'dismiss', this.dismiss);
            this._handleClick = this._handleClick.bind(this);
          },
          connected: function connected() {
            Component.dispatchComponentAddedEvent(this.element);
            this.closeButton = this.element.querySelector(this.closeSelector);
            this.closeButton.addEventListener('click', this._handleClick, false);
          },
          disconnected: function disconnected() {
            Component.dispatchComponentRemovedEvent(this.element);
            this.closeButton.removeEventListener('click', this._handleClick, false);
          },
          _handleClick: function _handleClick() {
            this.dismiss();
          },
          dismiss: function dismiss() {
            var dismissEvent = Component.dispatchCustomEvent('alertDismiss', this.element, {
              id: this.element.dataset.rvtAlert
            });
            if (!dismissEvent) return;
            this.element.remove();
          }
        };
      }
    }]);

    return Alert;
  }(Component);

  var Disclosure =
  function (_Component) {
    _inherits(Disclosure, _Component);

    function Disclosure() {
      _classCallCheck(this, Disclosure);

      return _possibleConstructorReturn(this, _getPrototypeOf(Disclosure).apply(this, arguments));
    }

    _createClass(Disclosure, null, [{
      key: "selector",
      get: function get() {
        return '[data-rvt-disclosure]';
      }
    }, {
      key: "methods",
      get: function get() {
        return {
          init: function init() {
            console.log('Disclosure::init');
            this.toggleElement = this.element.querySelector('[data-rvt-disclosure-toggle]');
            this.targetElement = this.element.querySelector('[data-rvt-disclosure-target]');
            this.isOpen = false;
            this.activeToggle = null;
            this.activeDisclosure = null;
            var icon = this.element.querySelector('svg');

            if (icon) {
              icon.setAttribute('focusable', 'false');
            }

            Component.bindMethodToDOMElement(this, 'open', this.open);
            Component.bindMethodToDOMElement(this, 'close', this.close);
            this._handleClick = this._handleClick.bind(this);
            this._handleKeydown = this._handleKeydown.bind(this);
          },
          connected: function connected() {
            Component.dispatchComponentAddedEvent(this.element);
            document.addEventListener('click', this._handleClick, false);
            document.addEventListener('keydown', this._handleKeydown, false);
          },
          disconnected: function disconnected() {
            Component.dispatchComponentRemovedEvent(this.element);
            document.removeEventListener('click', this._handleClick, false);
            document.removeEventListener('keydown', this._handleKeydown, false);
          },
          open: function open() {
            if (this.toggleElement.hasAttribute('disabled')) {
              return;
            } 


            var openEvent = Component.dispatchCustomEvent('disclosureOpen', this.toggleElement, {
              id: this.toggleElement.dataset['toggle']
            }); 

            if (!openEvent) return; 

            this.isOpen = true;
            this.toggleElement.setAttribute('aria-expanded', 'true'); 

            this.targetElement.removeAttribute('hidden'); 

            this.activeToggle = this.toggleElement;
            this.activeDisclosure = this.targetElement;
          },
          close: function close() {
            if (!this.activeToggle) return;
            var closeEvent = Component.dispatchCustomEvent('disclosureClose', this.toggleElement, {
              id: this.toggleElement.dataset['toggle']
            });
            if (!closeEvent) return;
            this.isOpen = false;
            this.activeToggle.setAttribute('aria-expanded', 'false');
            this.activeDisclosure.setAttribute('hidden', ''); 

            this.activeToggle = null;
            this.activeDisclosure = null;
          },
          _handleClick: function _handleClick(event) {
            var toggle = event.target.closest('[data-rvt-disclosure-toggle]'); 

            if (this.targetElement.contains(event.target)) return; 

            if (!toggle && this.activeToggle !== null) {
              this.close();
              return;
            } 


            if (toggle !== this.toggleElement || this.isOpen) {
              this.close();
            } else {
              this.open();
            }
          },
          _shouldHandleKeydown: function _shouldHandleKeydown(event) {
            if (!this.element.contains(event.target)) return false; 

            var disclosure = event.target.closest('[data-rvt-disclosure]');
            if (disclosure !== this.element) return false;
            return true;
          },
          _handleKeydown: function _handleKeydown(event) {
            if (!this._shouldHandleKeydown(event)) return;

            switch (event.keyCode) {
              case keyCodes.escape:
                {
                  if (!this.activeToggle) return; 

                  this.close();
                  this.toggleElement.focus();

                  this.activeToggle = null;
                  break;
                }
            }
          }
        };
      }
    }]);

    return Disclosure;
  }(Component);


  var Dropdown =
  function (_Component) {
    _inherits(Dropdown, _Component);

    function Dropdown() {
      _classCallCheck(this, Dropdown);

      return _possibleConstructorReturn(this, _getPrototypeOf(Dropdown).apply(this, arguments));
    }

    _createClass(Dropdown, null, [{
      key: "selector",
      get: function get() {
        return '[data-rvt-dropdown]';
      }
    }, {
      key: "methods",
      get: function get() {
        return {
          init: function init() {
            console.log('Dropdown::init()');
            this.disclosureAttribute = '[data-rvt-dropdown]';
            this.toggleAttribute = '[data-rvt-dropdown-toggle]';
            this.toggleDataProperty = 'dropdownToggle';
            this.targetAttribute = '[data-rvt-dropdown-menu]';
            this.openEventName = 'dropdownOpen';
            this.closeEventNane = 'dropdownClose';
            this.toggleElement = this.element.querySelector('[data-rvt-dropdown-toggle]');
            this.targetElement = this.element.querySelector('[data-rvt-dropdown-menu]');
            this.isOpen = false;
            this.activeToggle = null;
            this.activeDisclosure = null;
            var icon = this.element.querySelector('svg');

            if (icon) {
              icon.setAttribute('focusable', 'false');
            }

            Component.bindMethodToDOMElement(this, 'open', this.open);
            Component.bindMethodToDOMElement(this, 'close', this.close);
            this._handleClick = this._handleClick.bind(this);
            this._handleKeydown = this._handleKeydown.bind(this);
            this.focusableElements = 'a[href], area[href], input:not([disabled]), select:not([disabled]), textarea:not([disabled]), button:not([disabled]), [tabindex="0"]';
          },
          connected: function connected() {
            Component.dispatchComponentAddedEvent(this.element);
            document.addEventListener('click', this._handleClick, false);
            document.addEventListener('keydown', this._handleKeydown, false);
          },
          disconnected: function disconnected() {
            Component.dispatchComponentRemovedEvent(this.element);
            document.removeEventListener('click', this._handleClick, false);
            document.removeEventListener('keydown', this._handleKeydown, false);
          },
          open: function open() {
            if (this.toggleElement.hasAttribute('disabled')) {
              return;
            } 


            var openEvent = Component.dispatchCustomEvent('disclosureOpen', this.toggleElement, {
              id: this.toggleElement.dataset['toggle']
            }); 

            if (!openEvent) return; 

            this.isOpen = true;
            this.toggleElement.setAttribute('aria-expanded', 'true'); 

            this.targetElement.removeAttribute('hidden'); 

            this.activeToggle = this.toggleElement;
            this.activeDisclosure = this.targetElement;
          },
          close: function close() {
            if (!this.activeToggle) return;
            var closeEvent = Component.dispatchCustomEvent('dropdownClose', this.toggleElement, {
              id: this.toggleElement.dataset['toggle']
            });
            if (!closeEvent) return;
            this.isOpen = false;
            this.activeToggle.setAttribute('aria-expanded', 'false');
            this.activeDisclosure.setAttribute('hidden', ''); 

            this.activeToggle = null;
            this.activeDisclosure = null;
          },
          _setUpMenu: function _setUpMenu(menu) {
            var menuObject = {}; 

            var menuFocusables = nodeListToArray(menu.querySelectorAll(this.focusableElements)); 

            menuObject.all = menuFocusables; 

            menuObject.first = menuFocusables[0]; 

            menuObject.last = menuFocusables[menuFocusables.length - 1];
            return menuObject;
          },
          _handleClick: function _handleClick(event) {
            var toggle = event.target.closest('[data-rvt-dropdown-toggle]'); 

            if (this.targetElement.contains(event.target)) return; 

            if (!toggle && this.activeToggle !== null) {
              this.close();
              return;
            } 


            if (toggle !== this.toggleElement || this.isOpen) {
              this.close();
            } else {
              this.open();
            }
          },
          _shouldHandleKeydown: function _shouldHandleKeydown(event) {
            if (!this.element.contains(event.target)) return false; 

            var disclosure = event.target.closest('[data-rvt-dropdown]');
            if (disclosure !== this.element) return false;
            return true;
          },
          _handleKeydown: function _handleKeydown(event) {
            if (!this._shouldHandleKeydown(event)) return;

            switch (event.keyCode) {
              case keyCodes.escape:
                {
                  if (!this.activeToggle) return; 

                  this.close();
                  this.toggleElement.focus();

                  this.activeToggle = null;
                  break;
                }

              case keyCodes.down:
                {
                  if (!this.isOpen) {
                    this.open();
                  } else {
                    var _currentMenu = this._setUpMenu(this.targetElement);

                    _currentMenu.first.focus();
                  }


                  if (!this.targetElement.contains(event.target)) break;

                  var currentMenu = this._setUpMenu(this.targetElement);

                  var currentIndex;

                  for (var i = 0; i < currentMenu.all.length; i++) {
                    if (event.target == currentMenu.all[i]) {
                      currentIndex = i;
                    }
                  }

                  var nextItem = currentMenu.all[currentIndex + 1];

                  if (!nextItem) {
                    currentMenu.first.focus();
                    return;
                  }

                  nextItem.focus();
                  break;
                }

              case keyCodes.up:
                {
                  event.preventDefault();

                  if (!this.targetElement.contains(event.target)) break;

                  var _currentMenu2 = this._setUpMenu(this.targetElement);

                  var _currentIndex;

                  for (var _i = 0; _i < _currentMenu2.all.length; _i++) {
                    if (event.target == _currentMenu2.all[_i]) {
                      _currentIndex = _i;
                    }
                  }

                  var previousItem = _currentMenu2.all[_currentIndex - 1];

                  if (!previousItem && _currentMenu2.last !== undefined) {
                    _currentMenu2.last.focus();

                    return;
                  }

                  previousItem.focus();
                  break;
                }

              case keyCodes.tab:
                {
                  if (!this.targetElement.contains(event.target)) break;

                  var _currentMenu3 = this._setUpMenu(this.targetElement); 


                  if (document.activeElement == _currentMenu3.last && !event.shiftKey) {
                    this.close();
                    return;
                  }

                  break;
                }
            }
          }
        };
      }
    }]);

    return Dropdown;
  }(Component);

  var FileInput =
  function (_Component) {
    _inherits(FileInput, _Component);

    function FileInput() {
      _classCallCheck(this, FileInput);

      return _possibleConstructorReturn(this, _getPrototypeOf(FileInput).apply(this, arguments));
    }

    _createClass(FileInput, null, [{
      key: "selector",
      get: function get() {
        return '[data-rvt-file-input]';
      }
    }, {
      key: "methods",
      get: function get() {
        return {
          init: function init() {
            console.log('FileInput::init()');
            this.wrapperAttribute = 'data-rvt-file-input';
            this.inputAttribute = 'data-rvt-file-input-button';
            this.previewAttribute = 'data-rvt-file-input-preview';
            this.previewText = this.element.querySelector("[".concat(this.previewAttribute, "]")).textContent;
            this._handleChange = this._handleChange.bind(this);
          },
          connected: function connected() {
            Component.dispatchComponentAddedEvent(this.element);
          },
          disconnected: function disconnected() {
            Component.dispatchComponentRemovedEvent(this.element);
          },
          _sanitizeHTML: function _sanitizeHTML(str) {
            var temp = document.createElement('div');
            temp.textContent = str;
            return temp.innerHTML;
          },
          _buildSingleFile: function _buildSingleFile(input) {
            var singleFileItem = document.createElement('span');

            var singleFileName = this._sanitizeHTML(input.files[0].name); 


            singleFileItem.textContent = singleFileName; 

            return singleFileItem;
          },
          _buildMultipleFiles: function _buildMultipleFiles(input) {
            var fileCount = document.createElement('span');
            fileCount.textContent = input.files.length + ' files selected';
            return fileCount;
          },
          onChange: function onChange(event) {
            this._handleChange();
          },
          _handleChange: function _handleChange() {
            var uploadInput = this.element.querySelector("[".concat(this.inputAttribute, "]")); 

            var uploadPreview = this.element.querySelector("[".concat(this.previewAttribute, "]")); 

            if (uploadInput.files.length > 0) {
              var attachedEvent = Component.dispatchCustomEvent('fileAttached', this.element, {
                id: this.element.dataset.rvtFileInput,
                files: Array.from(uploadInput.files).map(function (f) {
                  return f.name;
                })
              });
              if (!attachedEvent) return; 

              uploadPreview.innerHTML = '';

              uploadInput.files.length > 1 ? uploadPreview.appendChild(this._buildMultipleFiles(uploadInput)) : uploadPreview.appendChild(this._buildSingleFile(uploadInput));
            } else {
              uploadPreview.innerHTML = this.previewText;
            }
          }
        };
      }
    }]);

    return FileInput;
  }(Component);

  var Modal =
  function (_Component) {
    _inherits(Modal, _Component);

    function Modal() {
      _classCallCheck(this, Modal);

      return _possibleConstructorReturn(this, _getPrototypeOf(Modal).apply(this, arguments));
    }

    _createClass(Modal, null, [{
      key: "selector",
      get: function get() {
        return '[data-rvt-modal]';
      }
    }, {
      key: "methods",
      get: function get() {
        return {
          init: function init() {
            console.log('Modal::init()');
            this.openOnInit = this.element.hasAttribute('data-rvt-modal-open');
            this.dialog = this.element.hasAttribute('data-rvt-modal-dialog');
            this.modalAttribute = 'data-rvt-modal';
            this.modalSelector = "[".concat(this.modalAttribute, "]");
            this.modalDataValue = this.element.getAttribute('data-rvt-modal');
            this.innerModalAttribute = 'data-rvt-modal-inner';
            this.innerModalSelector = "[".concat(this.innerModalAttribute, "]");
            this.openAttribute = 'data-rvt-modal-trigger';
            this.openSelector = "[".concat(this.openAttribute, "]");
            this.openButton = document.querySelector("[".concat(this.openAttribute, "=\"").concat(this.modalDataValue, "\"]"));
            this.closeAttribute = 'data-rvt-modal-close';
            this.closeSelector = "[".concat(this.closeAttribute, "]"); 

            this.focusElements = 'a[href], area[href], input:not([disabled]), select:not([disabled]), textarea:not([disabled]), button:not([disabled]), [tabindex="-1"]'; 

            Component.bindMethodToDOMElement(this, 'open', this.open);
            Component.bindMethodToDOMElement(this, 'close', this.close);
            Component.bindMethodToDOMElement(this, 'focusTrigger', this.focusTrigger);
            Component.bindMethodToDOMElement(this, 'focusModal', this.focusModal);
            this._handleClick = this._handleClick.bind(this);
            this._handleKeydown = this._handleKeydown.bind(this);
          },
          connected: function connected() {
            Component.dispatchComponentAddedEvent(this.element);

            if (!this.openOnInit) {
              this.element.setAttribute('hidden', '');
            } 


            document.addEventListener('click', this._handleClick, false);
            document.addEventListener('keydown', this._handleKeydown, false);
          },
          disconnected: function disconnected() {
            Component.dispatchComponentRemovedEvent(this.element);
            document.removeEventListener('click', this._handleClick, false);
            document.removeEventListener('keydown', this._handleKeydown, false);
          },
          _handleClick: function _handleClick(event) {
            event.target.closest(this.innerModalSelector) !== null ? event.clickedInModal = true : event.clickedInModal = false;

            if (event.clickedInModal) {
              event.stopPropagation();
            } 


            var triggerSelectors = "".concat(this.openSelector, ", ").concat(this.closeSelector, ", ").concat(this.modalSelector);
            var trigger = event.target.closest(triggerSelectors); 

            if (!trigger) return; 

            var triggerContent;

            if (trigger.getAttribute(this.openAttribute)) {
              triggerContent = trigger.getAttribute(this.openAttribute);
            } else if (trigger.getAttribute(this.closeAttribute)) {
              triggerContent = trigger.getAttribute(this.closeAttribute);
            } else {
              triggerContent = event.target.closest(this.modalSelector);
            } 


            if (!triggerContent) return;

            switch (trigger != null) {
              case trigger.hasAttribute(this.openAttribute):
                {
                  if (trigger.getAttribute(this.openAttribute) !== this.modalDataValue) {
                    this.close(); 

                    return;
                  }

                  this.open();
                  this.element.focus();
                  break;
                }

              case trigger.hasAttribute(this.closeAttribute):
                {
                  event.preventDefault(); 

                  if (trigger.getAttribute(this.closeAttribute) !== this.modalDataValue) return;
                  this.close();
                  if (this.openButton !== null) this.openButton.focus();
                  break;
                }

              case !event.clickedInModal:
                {
                  if (this.dialog) return; 

                  if (triggerContent.getAttribute(this.modalAttribute) !== this.modalDataValue) return;
                  this.close();
                  if (this.openButton !== null) this.openButton.focus();
                  break;
                }

              default:
                {
                  return;
                }
            }
          },
          _handleBackwardTab: function _handleBackwardTab(first, last, event) {
            if (document.activeElement === first || document.activeElement === this.element) {
              event.preventDefault();
              last.focus();
            }
          },
          _handleForwardTab: function _handleForwardTab(first, last, event) {
            if (document.activeElement === last) {
              event.preventDefault();
              first.focus();
            }
          },
          _handleKeydown: function _handleKeydown(event) {
            var currentModal = event.target.closest(this.modalSelector); 

            if (!currentModal) return;

            switch (event.keyCode) {
              case keyCodes.tab:
                {
                  var focusList = nodeListToArray(currentModal.querySelectorAll(this.focusElements));
                  var firstFocus = focusList[0];
                  var finalFocus = focusList[focusList.length - 1]; 

                  event.shiftKey ? this._handleBackwardTab(firstFocus, finalFocus, event) : this._handleForwardTab(firstFocus, finalFocus, event);
                  break;
                }

              case keyCodes.escape:
                {
                  if (this.dialog) return; 

                  if (currentModal.getAttribute(this.modalAttribute) !== this.modalDataValue) return;
                  this.close();
                  if (this.openButton !== null) this.openButton.focus();
                  break;
                }
            }
          },
          open: function open() {
            var openEvent = Component.dispatchCustomEvent('modalOpen', this.element, {
              id: this.element.dataset.rvtModal
            });
            if (!openEvent) return;
            this.element.removeAttribute('hidden');
            document.body.classList.add('rvt-modal-open');
          },
          close: function close() {
            var closeEvent = Component.dispatchCustomEvent('modalClose', this.element, {
              id: this.element.dataset.rvtModal
            });
            if (!closeEvent) return;
            this.element.setAttribute('hidden', '');
            document.body.classList.remove('rvt-modal-open');
          },
          focusTrigger: function focusTrigger() {
            if (!this.openButton) {
              throw new Error("Could not find a modal trigger with the value of ".concat(this.modalDataValue));
            }

            this.openButton.focus();
          },
          focusModal: function focusModal() {
            if (!this.element) {
              throw new Error("Could not find a modal with the value of ".concat(this.modalDataValue));
            }

            this.element.focus();
          }
        };
      }
    }]);

    return Modal;
  }(Component);

  var Sidenav =
  function (_Component) {
    _inherits(Sidenav, _Component);

    function Sidenav() {
      _classCallCheck(this, Sidenav);

      return _possibleConstructorReturn(this, _getPrototypeOf(Sidenav).apply(this, arguments));
    }

    _createClass(Sidenav, null, [{
      key: "selector",
      get: function get() {
        return '[data-rvt-sidenav]';
      }
    }, {
      key: "methods",
      get: function get() {
        return {
          init: function init() {
            var _this = this;

            console.log('Sidenav::init');
            this.toggleAttribute = 'data-rvt-sidenav-toggle';
            this.toggleSelector = "[".concat(this.toggleAttribute, "]");
            this.listAttribute = 'data-rvt-sidenav-list';
            this.listSelector = "[".concat(this.listAttribute, "]");
            this.openAllOnInit = this.element.hasAttribute('data-rvt-sidenav-open-all');
            var menuToggles = nodeListToArray(this.element.querySelectorAll(this.toggleSelector));
            var childMenus = nodeListToArray(this.element.querySelectorAll(this.listSelector));
            menuToggles.forEach(function (element, index) {
              element.setAttribute('aria-haspopup', 'true'); 

              if (_this.openAllOnInit) {
                element.setAttribute('aria-expanded', 'true');
                childMenus[index].removeAttribute('hidden');
                return;
              } 


              if (element.getAttribute('aria-expanded') === 'true') {
                childMenus[index].removeAttribute('hidden');
              } else {
                element.setAttribute('aria-expanded', 'false');
                childMenus[index].setAttribute('hidden', '');
              }
            }); 

            Component.bindMethodToDOMElement(this, 'open', this.open);
            Component.bindMethodToDOMElement(this, 'close', this.close);
            this._handleClick = this._handleClick.bind(this);
          },
          connected: function connected() {
            Component.dispatchComponentAddedEvent(this.element);
          },
          disconnected: function disconnected() {
            Component.dispatchComponentRemovedEvent(this.element);
          },
          onClick: function onClick(event) {
            this._handleClick(event);
          },
          _handleClick: function _handleClick(event) {
            var toggleButton = event.target.closest(this.toggleSelector); 

            if (!toggleButton) return;
            var toggleId = toggleButton.dataset.rvtSidenavToggle;
            var targetList = this.element.querySelector("[".concat(this.listAttribute, "=\"").concat(toggleId, "\"]")); 

            if (!targetList || targetList.getAttribute(this.listAttribute) === '') {
              return;
            }

            targetList.hasAttribute('hidden') ? this.open(toggleId) : this.close(toggleId);
          },
          open: function open(toggleId) {
            var toggle = this.element.querySelector("[".concat(this.toggleAttribute, "=\"").concat(toggleId, "\"]"));
            var targetList = this.element.querySelector("[".concat(this.listAttribute, "=\"").concat(toggleId, "\"]"));

            if (!toggle || !targetList) {
              console.warn("No such toggle '".concat(toggleId, "' in Sidenav.open()"));
              return;
            }

            var openEvent = Component.dispatchCustomEvent('sidenavListOpened', toggle, {
              id: toggleId
            });
            if (!openEvent) return;
            toggle.setAttribute('aria-expanded', 'true');
            targetList.removeAttribute('hidden');
          },
          close: function close(toggleId) {
            var toggle = this.element.querySelector("[".concat(this.toggleAttribute, "=\"").concat(toggleId, "\"]"));
            var targetList = this.element.querySelector("[".concat(this.listAttribute, "=\"").concat(toggleId, "\"]"));

            if (!toggle || !targetList) {
              console.warn("No such toggle '".concat(toggleId, "' in Sidenav.close()"));
              return;
            }

            var closeEvent = Component.dispatchCustomEvent('sidenavListClosed', toggle, {
              id: toggleId
            });
            if (!closeEvent) return;
            toggle.setAttribute('aria-expanded', 'false');
            targetList.setAttribute('hidden', '');
          }
        };
      }
    }]);

    return Sidenav;
  }(Component);

  var Tabs =
  function (_Component) {
    _inherits(Tabs, _Component);

    function Tabs() {
      _classCallCheck(this, Tabs);

      return _possibleConstructorReturn(this, _getPrototypeOf(Tabs).apply(this, arguments));
    }

    _createClass(Tabs, null, [{
      key: "selector",
      get: function get() {
        return '[data-rvt-tabs]';
      }
    }, {
      key: "methods",
      get: function get() {
        return {
          init: function init() {
            var _this = this;

            console.log('Tabs::init()');
            this.tablist = this.element.querySelector('[role="tablist"]');
            this.tabAttribute = 'data-rvt-tab';
            this.tabSelector = "[".concat(this.tabAttribute, "]");
            this.tabs = nodeListToArray(this.element.querySelectorAll(this.tabSelector));
            this.panelAttribute = 'data-rvt-tab-panel';
            this.panelSelector = "[".concat(this.panelAttribute, "]");
            this.panels = nodeListToArray(this.element.querySelectorAll(this.panelSelector)); 

            var initialPanel;
            this.panels.forEach(function (panel, index) {
              if (panel.hasAttribute('data-rvt-tab-init')) {
                initialPanel = panel;
              } else {
                _this.tabs[index].setAttribute('tabindex', '-1');
              }
            }); 

            this.openOnInit = initialPanel.getAttribute(this.panelAttribute) || this.panels[0].getAttribute(this.panelAttribute);
            console.log(this.openOnInit); 

            Component.bindMethodToDOMElement(this, 'activateTab', this.activateTab);
            this._handleClick = this._handleClick.bind(this);
            this._handleKeydown = this._handleKeydown.bind(this);
          },
          connected: function connected() {
            Component.dispatchComponentAddedEvent(this.element);
            this.activateTab(this.openOnInit);
            this.tablist.addEventListener('click', this._handleClick, false);
            this.tablist.addEventListener('keydown', this._handleKeydown, false);
          },
          disconnected: function disconnected() {
            Component.dispatchComponentRemovedEvent(this.element);
            this.tablist.removeEventListener('click', this._handleClick, false);
            this.tablist.removeEventListener('keydown', this._handleKeydown, false);
          },
          _handleClick: function _handleClick(event) {
            var currentTab = event.target.closest(this.tabSelector); 

            if (!currentTab) return; 

            var tabId = currentTab.getAttribute(this.tabAttribute); 

            this.activateTab(tabId);
          },
          _handleKeydown: function _handleKeydown(event) {
            var currentTab = event.target.closest(this.tabSelector); 

            if (!currentTab) return; 

            var nextTab = this.tabs.indexOf(currentTab) + 1;
            var prevTab = this.tabs.indexOf(currentTab) - 1;

            switch (event.keyCode) {
              case keyCodes.right :
                {
                  !this.tabs[nextTab] ? this.tabs[0].focus() : this.tabs[nextTab].focus();
                  break;
                }

              case keyCodes.down:
                {
                  !this.tabs[nextTab] ? this.tabs[0].focus() : this.tabs[nextTab].focus();
                  break;
                }

              case keyCodes.left:
                {
                  !this.tabs[prevTab] ? this.tabs[this.tabs.length - 1].focus() : this.tabs[prevTab].focus();
                  break;
                }

              case keyCodes.up:
                {
                  !this.tabs[prevTab] ? this.tabs[this.tabs.length - 1].focus() : this.tabs[prevTab].focus();
                  break;
                }

              case keyCodes.end:
                {
                  this.tabs[this.tabs.length - 1].focus();
                  break;
                }

              case keyCodes.home:
                {
                  this.tabs[0].focus();
                  break;
                }
            }
          },
          activateTab: function activateTab(tabId) {
            var _this2 = this;

            var tab = this.element.querySelector("[".concat(this.panelAttribute, "=\"").concat(tabId, "\"]"));

            if (!tab) {
              console.warn("No such tab '".concat(tabId, "' in Tabs.activateTab()"));
              return;
            }

            var activationEvent = Component.dispatchCustomEvent('tabActivated', tab, {
              id: tab.dataset.rvtTabPanel
            });
            if (!activationEvent) return;
            var trigger = this.element.querySelector("[".concat(this.tabAttribute, "=\"").concat(tab.dataset.rvtTabPanel, "\"]"));
            this.panels.forEach(function (panel, index) {
              if (panel.getAttribute(_this2.panelAttribute) !== tab.dataset.rvtTabPanel) {
                panel.setAttribute('hidden', '');

                _this2.tabs[index].setAttribute('aria-selected', 'false');

                _this2.tabs[index].setAttribute('tabindex', '-1');
              }
            }); 

            tab.removeAttribute('hidden');
            trigger.setAttribute('aria-selected', 'true');
            trigger.removeAttribute('tabindex');
          }
        };
      }
    }]);

    return Tabs;
  }(Component);


  function init() {
    console.log('Auto-initializing all Rivet components...');
    Accordion.initAll();
    Alert.initAll();
    Disclosure.initAll();
    Dropdown.initAll();
    FileInput.initAll();
    Modal.initAll();
    Sidenav.initAll();
    Tabs.initAll();
  }

  exports.Accordion = Accordion;
  exports.Alert = Alert;
  exports.Disclosure = Disclosure;
  exports.Dropdown = Dropdown;
  exports.FileInput = FileInput;
  exports.Modal = Modal;
  exports.Sidenav = Sidenav;
  exports.Tabs = Tabs;
  exports.init = init;

  return exports;

}({}));
